@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2000, 01  
+ * Copyright (c) 1997, 2000, 01, 02
  *    Motoyuki Kasahara
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,18 +20,19 @@
 #include "config.h"
 #endif
 
+#include <sys/types.h>
 
 /*
  * Function declarations.
  */
 #ifdef __STDC__
-ssize_t read_all(int, void *, size_t);
-ssize_t write_all(int, const void *, size_t);
-ssize_t write_string_all(int, const void *);
+int read_all(int, int, void *, size_t);
+int write_all(int, int, const void *, size_t);
+int write_string_all(int, int, const char *);
 #else /* not __STDC__ */
-ssize_t read_all();
-ssize_t write_all();
-ssize_t write_string_all();
+int read_all();
+int write_all();
+int write_string_all();
 #endif /* not __STDC__ */
 
 #endif /* not IOALL_H */
