@@ -17,7 +17,7 @@
  * This program requires the following Autoconf macros:
  *   AC_C_CONST
  *   AC_HEADER_STDC
- *   AC_CHECK_HEADERS(string.h, memory.h, stdlib.h, netinet6/in6.h)
+ *   AC_CHECK_HEADERS(string.h, memory.h, stdlib.h)
  *   AC_CHECK_FUNCS(inet_pton strchr)
  *
  *   and HAVE_STRUCT_IN6_ADDR check.
@@ -92,6 +92,17 @@ static void ipv6_net_mask_length_to_address();
 static const char *strnopbrk();
 #endif
 
+
+/*
+ * Prefix of IPv4 mapped IPv6 address.
+ */
+static const unsigned char ipv4_mapped_address_prefix[] = {
+    0x00, 0x00, 0x00, 0x00,
+    0x00, 0x00, 0x00, 0x00,
+    0x00, 0x00, 0xff, 0xff
+};
+
+
 /*
  * Initialize `permission'.
  */
@@ -134,7 +145,7 @@ finalize_permission(permission)
  * If `permission' is a valid IPv4/IPv6 prefix or address, it is recognized
  * as it is.  Otherwise `pattern' is recognized as a host name.
  *
- * If succeeded, 0 is returned.  Otherwise, -1 is returned.
+ * The function returns 0 upon successful, -1 otherwise.
  */
 int
 add_permission(permission, pattern)
@@ -151,7 +162,7 @@ add_permission(permission, pattern)
     int i;
 
     /*
-     * Allocate memeories for new node and a host name.
+     * Allocate memories for a new node and a host name.
      */
     permission_p = (Permission *)malloc(sizeof(Permission));
     if (permission_p == NULL) {
@@ -193,7 +204,7 @@ add_permission(permission, pattern)
 	while ('0' <= *p && *p <= '9')
 	    p++;
 
-	if (*p == '\0') {
+	if (*p == '\0' && p != slash + 1) {
 	    net_mask_length = atoi(slash + 1);
 	    *slash = '\0';
 	} else {
@@ -214,7 +225,7 @@ add_permission(permission, pattern)
      */
     if (strchr(copied_pattern, ':') != NULL) {
 	/*
-	 * The pattern seems to be IPv6 address.
+	 * The pattern seems to be an IPv6 address.
 	 */
 	if (inet_pton(AF_INET6, copied_pattern, &permission_p->address) != 1) {
 	    syslog(LOG_ERR, "invalid IPv6 adderss: %s", copied_pattern);
@@ -237,15 +248,13 @@ add_permission(permission, pattern)
 
     } else if (strnopbrk(copied_pattern, "0123456789.") == NULL) {
 	/*
-	 * The pattern seems to be IPv4 address.
+	 * The pattern seems to be an IPv4 address.
 	 */
 	if (inet_pton(AF_INET, copied_pattern, &ipv4_address) != 1) {
 	    syslog(LOG_ERR, "invalid IPv4 adderss: %s", copied_pattern);
 	    goto failed;
 	}
-	memset(permission_p->address.s6_addr, 0, 10);
-	*((unsigned char *)permission_p->address.s6_addr + 10) = 0xff;
-	*((unsigned char *)permission_p->address.s6_addr + 11) = 0xff;
+	memcpy(permission_p->address.s6_addr, ipv4_mapped_address_prefix, 12);
 	memcpy((unsigned char *)permission_p->address.s6_addr + 12,
 	    &ipv4_address.s_addr, 4);
 
@@ -266,7 +275,7 @@ add_permission(permission, pattern)
 
     } else {
 	/*
-	 * The pattern seems to be host name.
+	 * The pattern seems to be a host name.
 	 */
 	if (slash != NULL)
 	    *slash = '/';
@@ -276,7 +285,7 @@ add_permission(permission, pattern)
     }
 
     /*
-     * Check address/netmask pair, if the given pattern has address.
+     * Check address/netmask pair, if the given pattern is address.
      */
     if (permission_p->host_name == NULL) {
 	for (i = 0; i < 16; i++) {
@@ -333,6 +342,7 @@ ipv6_net_mask_length_to_address(length, address)
 	*address_p++ = 0xff;
 	rest_length -= 8;
     }
+
     if (0 < rest_length)
 	*address_p = mod8_masks[rest_length];
 }
@@ -414,9 +424,7 @@ test_permission(permission, host_name, address_string, function)
     if (inet_pton(AF_INET6, address_buffer, &ipv6_address) == 1) {
 	/* nothing to be done */;
     } else if (inet_pton(AF_INET, address_buffer, &ipv4_address) == 1) {
-	memset(ipv6_address.s6_addr, 0, 16);
-	*((unsigned char *)ipv6_address.s6_addr + 10) = 0xff;
-	*((unsigned char *)ipv6_address.s6_addr + 11) = 0xff;
+	memcpy(ipv6_address.s6_addr, ipv4_mapped_address_prefix, 12);
 	memcpy((unsigned char *)ipv6_address.s6_addr + 12,
 	    &ipv4_address.s_addr, 4);
     } else {
