@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 1997, 1998  Motoyuki Kasahara
+ * Copyright (c) 1997, 98, 2000, 01  
+ *    Motoyuki Kasahara
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,17 +20,23 @@
 #include "config.h"
 #endif
 
+#include <sys/types.h>
+#include <netinet/in.h>
+
+#include "dummyin6.h"
 
 /*
  * Access permission list.
  */
-typedef struct permission {
+typedef struct Permission_Struct Permission;
+struct Permission_Struct {
     int not;			/* inverse permission */
-    char *hostname;		/* hostname pattern */
-    int address;		/* address */
-    int netmask;		/* netmask */
-    struct permission *next;	/* pointer to next node */
-} Permission;
+    char *host_name;		/* host name pattern */
+    struct in6_addr address;	/* address */
+    struct in6_addr net_mask;	/* net mask */
+    char *scope_name;		/* scope name pattern */
+    Permission *next;		/* pointer to next node */
+};
 
 
 
@@ -38,7 +45,7 @@ typedef struct permission {
  */
 #ifdef __STDC__
 void initialize_permission(Permission *);
-void clear_permission(Permission *);
+void finalize_permission(Permission *);
 int add_permission(Permission *, const char *);
 int test_permission(const Permission *, const char *, const char *,
     int (*func)(const char *, const char *));
@@ -46,7 +53,7 @@ int count_permission(const Permission *);
 
 #else /* not __STDC__ */
 void initialize_permission();
-void clear_permission();
+void finalize_permission();
 int add_permission();
 int test_permission();
 int count_permission();
