@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 1997  Motoyuki Kasahara
+ * Copyright (c) 1997, 2000, 01  
+ *    Motoyuki Kasahara
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,16 +20,21 @@
 #include "config.h"
 #endif
 
+#include <sys/types.h>
+
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 
 /*
  * Function declarations.
  */
 #ifdef __STDC__
-int set_server_port(int, int);
-int wait_server_port(int);
+int set_server_ports(int, int, fd_set *, int *);
+int wait_server_ports(fd_set *, fd_set *, int *);
 #else /* not __STDC__ */
-int set_server_port();
-int wait_server_port();
+int set_server_ports();
+int wait_server_ports();
 #endif /* not __STDC__ */
 
 #endif /* not SERVERPORT_H */
