@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 1999  Motoyuki Kasahara
+ * Copyright (c) 1999, 2000, 01  
+ *    Motoyuki Kasahara
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,7 +25,7 @@
  * Ticket Stock.
  */
 typedef struct {
-    char *filename;		/* lock filename */
+    char *file_name;		/* lock file name */
     int file;			/* lock file descriptor */
     int count;			/* the number of tickets */
     int lock_spot;		/* locked spot (gotten ticket) */
@@ -39,14 +40,14 @@ void initialize_ticket_stock(Ticket_Stock *);
 int bind_ticket_stock(Ticket_Stock *, const char *, int);
 int get_ticket(Ticket_Stock *);
 int unget_ticket(Ticket_Stock *);
-void clear_ticket_stock(Ticket_Stock *);
+void finalize_ticket_stock(Ticket_Stock *);
 
 #else /* not __STDC__ */
 void initialize_ticket_stock();
 int bind_ticket_stock();
 int get_ticket();
 int unget_ticket();
-void clear_ticket_stock();
+void finalize_ticket_stock();
 #endif /* not __STDC__ */
 
 #endif /* not TICKET_H */
