@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 98, 99, 2000, 01  
+ * Copyright (c) 1997, 98, 99, 2000, 01, 02
  *    Motoyuki Kasahara
  *
  * This program is free software; you can redistribute it and/or modify
@@ -178,15 +178,11 @@ protocol_main()
 
     initialize_line_buffer(&line_buffer);
     bind_file_to_line_buffer(&line_buffer, accepted_in_file);
+    set_line_buffer_timeout(&line_buffer, idle_timeout);
     eb_initialize_hookset(&ndtp_hookset);
     set_common_hooks(&ndtp_hookset);
 
     for (;;) {
-	/*
-	 * Set an alarm.
-	 */
-	alarm(idle_timeout);
-	
 	/*
 	 * Read a line received from the client.
 	 */
@@ -206,8 +202,8 @@ protocol_main()
 	    *(line + NDTP_SHORT_BUFFER_LENGTH) = '\0';
 	    syslog(LOG_INFO, "too long request line: line=%s..., host=%s(%s)",
 		line, client_host_name, client_address);
-	    if (write_string_all(accepted_out_file, "$?\n") < 0)
-		return -1;
+	    if (write_string_all(accepted_out_file, idle_timeout, "$?\n") <= 0)
+		goto die;
 	    continue;
 	}
 	syslog(LOG_DEBUG, "debug: received line: %s", line);
@@ -223,17 +219,12 @@ protocol_main()
 	    *(line + 1) = '\0';
 	    syslog(LOG_INFO, "unknown command: command=%s, host=%s(%s)",
 		line, client_host_name, client_address);
-	    if (write_string_all(accepted_out_file, "$?\n") < 0)
+	    if (write_string_all(accepted_out_file, idle_timeout, "$?\n") <= 0)
 		goto die;
 
 	    continue;
 	}
 
-	/*
-	 * Set an alarm, again.
-	 */
-	alarm(idle_timeout);
-
 	/*
 	 * Dispatch.
 	 */
@@ -250,13 +241,11 @@ protocol_main()
 	    break;
     }
 
-    alarm(0);
     finalize_line_buffer(&line_buffer);
     eb_finalize_hookset(&ndtp_hookset);
     return 0;
 
   die:
-    alarm(0);
     finalize_line_buffer(&line_buffer);
     eb_finalize_hookset(&ndtp_hookset);
     return -1;
@@ -285,8 +274,8 @@ command_L(command, argument)
      * Unset the current book.
      */
     if (current_book != NULL) {
-	eb_suspend(&current_book->book);
-	eb_suspend_appendix(&current_book->appendix);
+	eb_unset_subbook(&current_book->book);
+	eb_unset_appendix_subbook(&current_book->appendix);
 	current_book = NULL;
     }
     current_subbook_name[0] = '\0';
@@ -296,7 +285,7 @@ command_L(command, argument)
      * If `argument' is `nodict', unset the current book/subbook.
      */
     if (strcmp(argument, "nodict") == 0) {
-	if (write_string_all(accepted_out_file, "$*\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$*\n") <= 0)
 	    return -1;
 	syslog(LOG_INFO, "succeeded: command=%s, book=nodict, host=%s(%s)",
 	    command, client_host_name, client_address);
@@ -325,7 +314,7 @@ command_L(command, argument)
 	if (book_number_to_name(expanded_argument) < 0) {
 	    syslog(LOG_INFO, "unknown book: command=%s, book=%s, host=%s(%s)",
 		command, expanded_argument, client_host_name, client_address);
-	    if (write_string_all(accepted_out_file, "$&\n") < 0)
+	    if (write_string_all(accepted_out_file, idle_timeout, "$&\n") <= 0)
 		return -1;
 	    goto failed;
 	}
@@ -335,7 +324,7 @@ command_L(command, argument)
 	 * Unset the current book/subbook.
 	 */
 	if (strcmp(expanded_argument, "nodict") == 0) {
-	    if (write_string_all(accepted_out_file, "$*\n") < 0)
+	    if (write_string_all(accepted_out_file, idle_timeout, "$*\n") <= 0)
 		return -1;
 	    syslog(LOG_INFO, "succeeded: command=%s, book=nodict, host=%s(%s)",
 		command, client_host_name, client_address);
@@ -362,7 +351,7 @@ command_L(command, argument)
     if (current_book == NULL) {
 	syslog(LOG_INFO, "unknown book: command=%s, book=%s, host=%s(%s)",
 	    command, expanded_argument, client_host_name, client_address);
-	if (write_string_all(accepted_out_file, "$&\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$&\n") <= 0)
 	    return -1;
 	goto failed;
     }
@@ -373,7 +362,7 @@ command_L(command, argument)
     if (!current_book->permission_flag) {
 	syslog(LOG_ERR, "denied: command=%s, book=%s, host=%s(%s)",
 	    command, expanded_argument, client_host_name, client_address);
-	if (write_string_all(accepted_out_file, "$&\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$&\n") <= 0)
 	    return -1;
 	goto failed;
     }
@@ -386,7 +375,7 @@ command_L(command, argument)
 	&& get_ticket(&current_book->ticket_stock) < 0) {
 	syslog(LOG_INFO, "full of clients: command=%s, book=%s, host=%s(%s)",
 	    command, expanded_argument, client_host_name, client_address);
-	if (write_string_all(accepted_out_file, "$&\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$&\n") <= 0)
 	    return -1;
 	goto failed;
     }
@@ -398,7 +387,7 @@ command_L(command, argument)
     if (subbook < 0) {
 	syslog(LOG_INFO, "unknown book: command=%s, book=%s, host=%s(%s)",
 	    command, expanded_argument, client_host_name, client_address);
-	if (write_string_all(accepted_out_file, "$&\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$&\n") <= 0)
 	    return -1;
 	goto failed;
     }
@@ -411,7 +400,7 @@ command_L(command, argument)
 	syslog(LOG_INFO, "%s: command=%s, book=%s, host=%s(%s)",
 	    eb_error_message(error_code), command, expanded_argument,
 	    client_host_name, client_address);
-	if (write_string_all(accepted_out_file, "$&\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$&\n") <= 0)
 	    return -1;
 	goto failed;
     }
@@ -426,7 +415,7 @@ command_L(command, argument)
 	    syslog(LOG_INFO, "unknown appendix: command=%s, book=%s, \
 host=%s(%s)",
 		command, expanded_argument, client_host_name, client_address);
-	    if (write_string_all(accepted_out_file, "$&\n") < 0)
+	    if (write_string_all(accepted_out_file, idle_timeout, "$&\n") <= 0)
 		return -1;
 	    goto failed;
 	}
@@ -440,7 +429,7 @@ host=%s(%s)",
 	    syslog(LOG_INFO, "%s: command=%s, book=%s, host=%s(%s)",
 		eb_error_message(error_code), command, expanded_argument,
 		client_host_name, client_address);
-	    if (write_string_all(accepted_out_file, "$&\n") < 0)
+	    if (write_string_all(accepted_out_file, idle_timeout, "$&\n") <= 0)
 		return -1;
 	    goto failed;
 	}
@@ -455,7 +444,7 @@ host=%s(%s)",
 	syslog(LOG_INFO, "%s: command=%s, book=%s, host=%s(%s)",
 	    eb_error_message(error_code), command, expanded_argument,
 	    client_host_name, client_address);
-	if (write_string_all(accepted_out_file, "$&\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$&\n") <= 0)
 	    return -1;
 	goto failed;
     }
@@ -468,7 +457,7 @@ host=%s(%s)",
     /*
      * Send a response.
      */
-    if (write_string_all(accepted_out_file, "$*\n") < 0)
+    if (write_string_all(accepted_out_file, idle_timeout, "$*\n") <= 0)
 	return -1;
 
     syslog(LOG_INFO, "succeeded: command=%s, book=%s, host=%s(%s)",
@@ -480,8 +469,8 @@ host=%s(%s)",
      */
   failed:
     if (current_book != NULL) {
-	eb_suspend(&current_book->book);
-	eb_suspend_appendix(&current_book->appendix);
+	eb_unset_subbook(&current_book->book);
+	eb_unset_appendix_subbook(&current_book->appendix);
 	current_book = NULL;
     }
     return 0;
@@ -523,7 +512,7 @@ command_A(command, argument)
     /*
      * Send a success code.
      */
-    if (write_string_all(accepted_out_file, "$A\n") < 0)
+    if (write_string_all(accepted_out_file, idle_timeout, "$A\n") <= 0)
 	return -1;
 
     syslog(LOG_INFO, "succeeded: command=%s, host=%s(%s)",
@@ -547,7 +536,7 @@ command_u(command, argument)
     if (*argument != '\0') {
 	syslog(LOG_INFO, "too many arguments: command=%s, host=%s(%s)",
 	    command, client_host_name, client_address);
-	if (write_string_all(accepted_out_file, "$?\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$?\n") <= 0)
 	    return -1;
 	return 0;
     }
@@ -555,7 +544,7 @@ command_u(command, argument)
     /*
      * Send an empty list.
      */
-    if (write_string_all(accepted_out_file, "$U\n$$\n") < 0)
+    if (write_string_all(accepted_out_file, idle_timeout, "$U\n$$\n") <= 0)
 	return -1;
 
     syslog(LOG_INFO, "succeeded: command=%s, host=%s(%s)",
@@ -602,7 +591,7 @@ command_P(command, argument)
     if (current_book == NULL) {
 	syslog(LOG_INFO, "no current book: command=%s, host=%s(%s)",
 	    command, client_host_name, client_address);
-	if (write_string_all(accepted_out_file, "$<\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$<\n") <= 0)
 	    return -1;
 	return 0;
     }
@@ -614,14 +603,14 @@ command_P(command, argument)
     if (EB_MAX_WORD_LENGTH < word_length) {
 	syslog(LOG_INFO, "too long word: command=%s, host=%s(%s)", 
 	    command, client_host_name, client_address);
-	if (write_string_all(accepted_out_file, "$?\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$?\n") <= 0)
 	    return -1;
 	return 0;
     }
     if (*argument == '\0') {
 	syslog(LOG_INFO, "empty method name: command=%s, host=%s(%s)",
 	    command, client_host_name, client_address);
-	if (write_string_all(accepted_out_file, "$?\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$?\n") <= 0)
 	    return -1;
 	return 0;
     }
@@ -635,7 +624,7 @@ command_P(command, argument)
 	syslog(LOG_INFO, "bad search method: command=%s, method=%c, \
 host=%s(%s)",
 	    command, method, client_host_name, client_address);
-	if (write_string_all(accepted_out_file, "$?\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$?\n") <= 0)
 	    return -1;
 	return 0;
     }
@@ -722,7 +711,7 @@ host=%s(%s)",
 	    eb_error_message(error_code), command, current_book->name,
 	    current_subbook_name, method, argument + 1, client_host_name,
 	    client_address);
-	if (write_string_all(accepted_out_file, "$0\n$$\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$0\n$$\n") <= 0)
 	    return -1;
 	return 0;
     }
@@ -730,7 +719,7 @@ host=%s(%s)",
     /*
      * Get hit entries and send them to a client.
      */
-    if (write_string_all(accepted_out_file, "$0\n") < 0)
+    if (write_string_all(accepted_out_file, idle_timeout, "$0\n") <= 0)
 	return -1;
 
     hit_count = 0;
@@ -800,11 +789,13 @@ word=%s, host=%s(%s)",
 	    /*
 	     * Output the heading and text position.
 	     */
-	    if (write_string_all(accepted_out_file, heading) < 0)
+	    if (write_string_all(accepted_out_file, idle_timeout, heading)
+		<= 0)
 		return -1;
 	    sprintf(position_buffer, "\n%x:%x\n", hit->text.page,
 		hit->text.offset);
-	    if (write_string_all(accepted_out_file, position_buffer) < 0)
+	    if (write_string_all(accepted_out_file, idle_timeout,
+		position_buffer) <= 0)
 		return -1;
 
 	    /*
@@ -830,7 +821,7 @@ host=%s(%s)",
 	    current_subbook_name, method, argument + 1, client_host_name,
 	    client_address);
     }
-    if (write_string_all(accepted_out_file, "$$\n") < 0)
+    if (write_string_all(accepted_out_file, idle_timeout, "$$\n") <= 0)
 	return -1;
 
     syslog(LOG_INFO, "succeeded: command=%s, book=%s/%s, method=%c, \
@@ -870,7 +861,7 @@ command_F(command, argument)
     if (current_book == NULL) {
 	syslog(LOG_INFO, "no current book: command=%s, host=%s(%s)",
 	    command, client_host_name, client_address);
-	if (write_string_all(accepted_out_file, "$<\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$<\n") <= 0)
 	    return -1;
 	return 0;
     }
@@ -883,7 +874,7 @@ command_F(command, argument)
     if (!isxdigit(*argument) || *end_p != '\0') {
 	syslog(LOG_INFO, "bad page number: command=%s, page=%s, host=%s(%s)",
 	    command, argument, client_host_name, client_address);
-	if (write_string_all(accepted_out_file, "$?\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$?\n") <= 0)
 	    return -1;
 	return 0;
     }
@@ -898,7 +889,8 @@ command_F(command, argument)
 	    current_subbook_name, position.page, client_host_name,
 	    client_address);
 	memset(message + 2, '\0', EB_SIZE_PAGE);
-	if (write_all(accepted_out_file, message, EB_SIZE_PAGE + 3) < 0)
+	if (write_all(accepted_out_file, idle_timeout, message,
+	    EB_SIZE_PAGE + 3) <= 0)
 	    return -1;
 	return 0;
     }
@@ -910,7 +902,8 @@ command_F(command, argument)
 	    current_subbook_name, position.page, client_host_name,
 	    client_address);
 	memset(message + 2, '\0', EB_SIZE_PAGE);
-	if (write_all(accepted_out_file, message, EB_SIZE_PAGE + 3) < 0)
+	if (write_all(accepted_out_file, idle_timeout, message,
+	    EB_SIZE_PAGE + 3) <= 0)
 	    return -1;
 	return 0;
     }
@@ -918,7 +911,8 @@ command_F(command, argument)
     /*
      * Send the page to the client ("$F" + page + "\n").
      */
-    if (write_all(accepted_out_file, message, EB_SIZE_PAGE + 3) < 0)
+    if (write_all(accepted_out_file, idle_timeout, message,
+	EB_SIZE_PAGE + 3) <= 0)
 	return -1;
 
     syslog(LOG_INFO, "succeeded: command=%s, book=%s/%s, page=%d, host=%s(%s)",
@@ -954,7 +948,7 @@ command_S(command, argument)
     if (current_book == NULL) {
 	syslog(LOG_INFO, "no current book: command=%s, host=%s(%s)",
 	    command, client_host_name, client_address);
-	if (write_string_all(accepted_out_file, "$<\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$<\n") <= 0)
 	    return -1;
 	return 0;
     }
@@ -966,7 +960,7 @@ command_S(command, argument)
     if (!isxdigit(*argument) || *end_p != ':') {
 	syslog(LOG_INFO, "bad position: command=%s, position=%s, host=%s(%s)",
 	    command, argument, client_host_name, client_address);
-	if (write_string_all(accepted_out_file, "$?\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$?\n") <= 0)
 	    return -1;
 	return 0;
     }
@@ -979,7 +973,7 @@ command_S(command, argument)
     if (!isxdigit(*offset_p) || *end_p != '\0') {
 	syslog(LOG_INFO, "bad position: command=%s, position=%s, host=%s(%s)",
 	    command, argument, client_host_name, client_address);
-	if (write_string_all(accepted_out_file, "$?\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$?\n") <= 0)
 	    return -1;
 	return 0;
     }
@@ -1000,7 +994,7 @@ command_S(command, argument)
 host=%s(%s)",
 	    eb_error_message(error_code), command, current_book->name,
 	    current_subbook_name, argument, client_host_name, client_address);
-	if (write_string_all(accepted_out_file, "$1\n$$\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$1\n$$\n") <= 0)
 	    return -1;
 	return 0;
     }
@@ -1008,7 +1002,7 @@ host=%s(%s)",
     /*
      * Read text and send them to the client.
      */
-    if (write_string_all(accepted_out_file, "$1\n") < 0)
+    if (write_string_all(accepted_out_file, idle_timeout, "$1\n") <= 0)
 	return -1;
 
     rest_text_length = max_text_size;
@@ -1028,11 +1022,12 @@ host=%s(%s)",
 	rest_text_length -= actual_text_length;
 	if (character_code == EB_CHARCODE_ISO8859_1)
 	    iso8859_1_to_ascii_mem(text, actual_text_length);
-	if (write_all(accepted_out_file, text, actual_text_length) < 0)
+	if (write_all(accepted_out_file, idle_timeout, text,
+	    actual_text_length) <= 0)
 	    return -1;
     }
 
-    if (write_string_all(accepted_out_file, "\n$$\n") < 0)
+    if (write_string_all(accepted_out_file, idle_timeout, "\n$$\n") <= 0)
 	return -1;
 
     syslog(LOG_INFO, "succeeded: command=%s, book=%s/%s, position=%s, \
@@ -1061,7 +1056,7 @@ command_I(command, argument)
     if (*argument != '\0') {
 	syslog(LOG_INFO, "too many arguments: command=%s, host=%s(%s)",
 	    command, client_host_name, client_address);
-	if (write_string_all(accepted_out_file, "$?\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$?\n") <= 0)
 	    return -1;
 	return 0;
     }
@@ -1072,7 +1067,7 @@ command_I(command, argument)
     if (current_book == NULL) {
 	syslog(LOG_INFO, "no current book: command=%s, host=%s(%s)",
 	    command, client_host_name, client_address);
-	if (write_string_all(accepted_out_file, "$<\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$<\n") <= 0)
 	    return -1;
 	return 0;
     }
@@ -1080,32 +1075,34 @@ command_I(command, argument)
     /*
      * Send response data.
      */
-    if (write_string_all(accepted_out_file, "$I\n") < 0)
+    if (write_string_all(accepted_out_file, idle_timeout, "$I\n") <= 0)
 	return -1;
     if (eb_have_word_search(&current_book->book)) {
-	if (write_string_all(accepted_out_file, "IA 1\nIK 1\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "IA 1\nIK 1\n")
+	    <= 0)
 	    return -1;
     }
     if (eb_have_endword_search(&current_book->book)) {
-	if (write_string_all(accepted_out_file, "BA 1\nBK 1\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "BA 1\nBK 1\n")
+	    <= 0)
 	    return -1;
     }
     if (eb_menu(&current_book->book, &position) == EB_SUCCESS) {
 	sprintf(message, "HA %x\n", position.page);
-	if (write_string_all(accepted_out_file, message) < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, message) <= 0)
 	    return -1;
     }
     if (eb_copyright(&current_book->book, &position) == EB_SUCCESS) {
 	sprintf(message, "OK %x\n", position.page);
-	if (write_string_all(accepted_out_file, message) < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, message) <= 0)
 	    return -1;
     }
     if (eb_have_keyword_search(&current_book->book)) {
-	if (write_string_all(accepted_out_file, "JO 1\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "JO 1\n") <= 0)
 	    return -1;
     }
 
-    if (write_string_all(accepted_out_file, "$$\n") < 0)
+    if (write_string_all(accepted_out_file, idle_timeout, "$$\n") <= 0)
 	return -1;
 
     syslog(LOG_INFO, "succeeded: command=%s, book=%s/%s, host=%s(%s)",
@@ -1131,13 +1128,13 @@ command_v(command, argument)
     if (*argument != '\0') {
 	syslog(LOG_INFO, "too many arguments: command=%s, host=%s(%s)",
 	    command, client_host_name, client_address);
-	if (write_string_all(accepted_out_file, "$?\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$?\n") <= 0)
 	    return -1;
 	return 0;
     }
 
     sprintf(message, "$v%s version %s\n", program_name, program_version);
-    if (write_string_all(accepted_out_file, message) < 0)
+    if (write_string_all(accepted_out_file, idle_timeout, message) <= 0)
 	return -1;
 
     syslog(LOG_INFO, "succeeded: command=%s, host=%s(%s)",
@@ -1169,7 +1166,7 @@ command_T(command, argument)
     if (*argument != '\0') {
 	syslog(LOG_INFO, "too many arguments: command=%s, host=%s(%s)",
 	    command, client_host_name, client_address);
-	if (write_string_all(accepted_out_file, "$?\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$?\n") <= 0)
 	    return -1;
 	return 0;
     }
@@ -1196,11 +1193,12 @@ command_T(command, argument)
 	    sprintf(message, "%d\t%s\n", i, title);
 	    if (character_code == EB_CHARCODE_ISO8859_1)
 		iso8859_1_to_ascii_str(message);
-	    if (write_string_all(accepted_out_file, message) < 0)
+	    if (write_string_all(accepted_out_file, idle_timeout, message)
+		<= 0)
 		return -1;
 	}
     }
-    if (write_string_all(accepted_out_file, "$*\n") < 0)
+    if (write_string_all(accepted_out_file, idle_timeout, "$*\n") <= 0)
 	return -1;
 
     syslog(LOG_INFO, "succeeded: command=%s, host=%s(%s)",
@@ -1234,7 +1232,7 @@ command_t(command, argument)
     if (*argument != '\0') {
 	syslog(LOG_INFO, "too many arguments: command=%s, host=%s(%s)",
 	    command, client_host_name, client_address);
-	if (write_string_all(accepted_out_file, "$?\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$?\n") <= 0)
 	    return -1;
 	return 0;
     }
@@ -1267,11 +1265,12 @@ command_t(command, argument)
 		book->name, directory, 0, book->max_clients, idle_timeout);
 	    if (character_code == EB_CHARCODE_ISO8859_1)
 		iso8859_1_to_ascii_str(message);
-	    if (write_string_all(accepted_out_file, message) < 0)
+	    if (write_string_all(accepted_out_file, idle_timeout, message)
+		<= 0)
 		return -1;
 	}
     }
-    if (write_string_all(accepted_out_file, "$*\n") < 0)
+    if (write_string_all(accepted_out_file, idle_timeout, "$*\n") <= 0)
 	return -1;
 
     syslog(LOG_INFO, "succeeded: command=%s, host=%s(%s)",
@@ -1303,7 +1302,7 @@ command_X(command, argument)
 
     syslog(LOG_INFO, "unknown command: command=%s, host=%s(%s)",
 	command, client_host_name, client_address);
-    if (write_string_all(accepted_out_file, "$?\n") < 0)
+    if (write_string_all(accepted_out_file, idle_timeout, "$?\n") <= 0)
 	return -1;
 
     return 0;
@@ -1331,7 +1330,7 @@ command_XI(command, argument)
     if (*argument != '\0') {
 	syslog(LOG_INFO, "too many arguments: command=%s, host=%s(%s)",
 	    command, client_host_name, client_address);
-	if (write_string_all(accepted_out_file, "$?\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$?\n") <= 0)
 	    return -1;
 	return 0;
     }
@@ -1342,7 +1341,7 @@ command_XI(command, argument)
     if (current_book == NULL) {
 	syslog(LOG_INFO, "no current book: command=%s, host=%s(%s)",
 	    command, client_host_name, client_address);
-	if (write_string_all(accepted_out_file, "$<\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$<\n") <= 0)
 	    return -1;
 	return 0;
     }
@@ -1350,7 +1349,7 @@ command_XI(command, argument)
     /*
      * Send response data.
      */
-    if (write_string_all(accepted_out_file, "$I\n") < 0)
+    if (write_string_all(accepted_out_file, idle_timeout, "$I\n") <= 0)
 	return -1;
 
     error_code = eb_font_list(&current_book->book, font_list, &font_count);
@@ -1360,10 +1359,10 @@ command_XI(command, argument)
 	if (eb_font_height2(font_list[i], &height) != EB_SUCCESS)
 	    continue;
 	sprintf(message, "%d\n", height);
-	if (write_string_all(accepted_out_file, message) < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, message) <= 0)
 	    return -1;
     }
-    if (write_string_all(accepted_out_file, "$$\n") < 0)
+    if (write_string_all(accepted_out_file, idle_timeout, "$$\n") <= 0)
 	return -1;
 
     syslog(LOG_INFO, "succeeded: command=%s, book=%s/%s, host=%s(%s)",
@@ -1392,7 +1391,7 @@ command_XL(command, argument)
     if (current_book == NULL) {
 	syslog(LOG_INFO, "no current book: command=%s, host=%s(%s)",
 	    command, client_host_name, client_address);
-	if (write_string_all(accepted_out_file, "$<\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$<\n") <= 0)
 	    return -1;
 	goto failed;
     }
@@ -1411,7 +1410,7 @@ command_XL(command, argument)
 	syslog(LOG_INFO, "bad bitmap height: command=%s, bitmap-size=%s, \
 host=%s(%s)",
 	    command, argument, client_host_name, client_address);
-	if (write_string_all(accepted_out_file, "$?\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$?\n") <= 0)
 	    return -1;
 	goto failed;
     }
@@ -1420,7 +1419,7 @@ host=%s(%s)",
      * If height is `0', unset the bitmap size.
      */
     if (height == 0) {
-	if (write_string_all(accepted_out_file, "$*\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$*\n") <= 0)
 	    return -1;
 	syslog(LOG_INFO, "succeeded: command=%s, book=%s/%s, bitmap-size=0, \
 host=%s(%s)",
@@ -1451,7 +1450,7 @@ host=%s(%s)",
 bitmap-size=%d, host=%s(%s)",
 	    command, current_book->name, current_subbook_name, height,
 	    client_host_name, client_address);
-	if (write_string_all(accepted_out_file, "$&\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$&\n") <= 0)
 	    return -1;
 	goto failed;
     }
@@ -1465,7 +1464,7 @@ bitmap-size=%d, host=%s(%s)",
 host=%s(%s)",
 	    eb_error_message(error_code), command, current_book->name,
 	    current_subbook_name, height, client_host_name, client_address);
-	if (write_string_all(accepted_out_file, "$&\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$&\n") <= 0)
 	    return -1;
 	goto failed;
     }
@@ -1475,7 +1474,7 @@ host=%s(%s)",
     /*
      * Send a response.
      */
-    if (write_string_all(accepted_out_file, "$*\n") < 0)
+    if (write_string_all(accepted_out_file, idle_timeout, "$*\n") <= 0)
 	return -1;
     syslog(LOG_INFO, "succeeded: command=%s, book=%s/%s, bitmap-size=%d, \
 host=%s(%s)",
@@ -1522,7 +1521,7 @@ command_XB(command, argument)
     if (*argument != '\0') {
 	syslog(LOG_INFO, "too many arguments: command=%s, host=%s(%s)",
 	    command, client_host_name, client_address);
-	if (write_string_all(accepted_out_file, "$?\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$?\n") <= 0)
 	    return -1;
 	return 0;
     }
@@ -1533,7 +1532,7 @@ command_XB(command, argument)
     if (current_book == NULL) {
 	syslog(LOG_INFO, "no current book: command=%s, host=%s(%s)",
 	    command, client_host_name, client_address);
-	if (write_string_all(accepted_out_file, "$<\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$<\n") <= 0)
 	    return -1;
 	return 0;
     }
@@ -1542,12 +1541,12 @@ command_XB(command, argument)
 host=%s(%s)",
 	    command, current_book->name, current_subbook_name,
 	    client_host_name, client_address);
-	if (write_string_all(accepted_out_file, "$<\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$<\n") <= 0)
 	    return -1;
 	return 0;
     }
 
-    if (write_string_all(accepted_out_file, "$I\n") < 0)
+    if (write_string_all(accepted_out_file, idle_timeout, "$I\n") <= 0)
 	return -1;
 
     /*
@@ -1596,9 +1595,10 @@ host=%s(%s)",
 	    eb_bitmap_to_xbm(bitmap, width, current_font_height, xbm,
 		&xbm_length);
 	    sprintf(header, "$=h%04x\n", ch);
-	    if (write_string_all(accepted_out_file, header) < 0)
+	    if (write_string_all(accepted_out_file, idle_timeout, header) <= 0)
 		return -1;
-	    if (write_all(accepted_out_file, xbm, xbm_length) < 0)
+	    if (write_all(accepted_out_file, idle_timeout, xbm, xbm_length)
+		<= 0)
 		return -1;
 	    error_code = eb_forward_narrow_font_character(&current_book->book,
 		1, &ch);
@@ -1654,9 +1654,10 @@ host=%s(%s)",
 	    eb_bitmap_to_xbm(bitmap, width, current_font_height, xbm,
 		&xbm_length);
 	    sprintf(header, "$=z%04x\n", ch);
-	    if (write_string_all(accepted_out_file, header) < 0)
+	    if (write_string_all(accepted_out_file, idle_timeout, header) <= 0)
 		return -1;
-	    if (write_all(accepted_out_file, xbm, xbm_length) < 0)
+	    if (write_all(accepted_out_file, idle_timeout, xbm, xbm_length)
+		<= 0)
 		return -1;
 	    error_code = eb_forward_wide_font_character(&current_book->book,
 		1, &ch);
@@ -1668,7 +1669,7 @@ host=%s(%s)",
     /*
      * End of the response data.
      */
-    if (write_string_all(accepted_out_file, "$$\n") < 0)
+    if (write_string_all(accepted_out_file, idle_timeout, "$$\n") <= 0)
 	return -1;
 
     syslog(LOG_INFO, "succeeded: command=%s, book=%s/%s, bitmap-size=%d, \
@@ -1702,7 +1703,7 @@ command_Xb(command, argument)
     if (current_book == NULL) {
 	syslog(LOG_INFO, "no current book: command=%s, host=%s(%s)",
 	    command, client_host_name, client_address);
-	if (write_string_all(accepted_out_file, "$<\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$<\n") <= 0)
 	    return -1;
 	return 0;
     }
@@ -1711,7 +1712,7 @@ command_Xb(command, argument)
 host=%s(%s)",
 	    command, current_book->name, current_subbook_name, 
 	    client_host_name, client_address);
-	if (write_string_all(accepted_out_file, "$<\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$<\n") <= 0)
 	    return -1;
 	return 0;
     }
@@ -1727,7 +1728,7 @@ host=%s(%s)",
 	syslog(LOG_INFO, "bad bitmap height: command=%s, character=%s, \
 host=%s(%s)",
 	    command, argument, client_host_name, client_address);
-	if (write_string_all(accepted_out_file, "$?\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$?\n") <= 0)
 	    return -1;
 	return 0;
     }
@@ -1745,7 +1746,7 @@ character=%s, host=%s(%s)",
 	    eb_error_message(error_code), command, current_book->name,
 	    current_subbook_name, current_font_height, argument,
 	    client_host_name, client_address);
-	if (write_string_all(accepted_out_file, "$&\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$&\n") <= 0)
 	    return -1;
 	return 0;
     }
@@ -1766,7 +1767,7 @@ character=%s, host=%s(%s)",
 	    eb_error_message(error_code), command, current_book->name,
 	    current_subbook_name, current_font_height, argument,
 	    client_host_name, client_address);
-	if (write_string_all(accepted_out_file, "$&\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$&\n") <= 0)
 	    return -1;
 	return 0;
     }
@@ -1779,11 +1780,11 @@ character=%s, host=%s(%s)",
     /*
      * Send bitmap data of the character.
      */
-    if (write_string_all(accepted_out_file, "$I\n") < 0)
+    if (write_string_all(accepted_out_file, idle_timeout, "$I\n") <= 0)
 	return -1;
-    if (write_all(accepted_out_file, xbm, xbm_length) < 0)
+    if (write_all(accepted_out_file, idle_timeout, xbm, xbm_length) <= 0)
 	return -1;
-    if (write_string_all(accepted_out_file, "$$\n") < 0)
+    if (write_string_all(accepted_out_file, idle_timeout, "$$\n") <= 0)
 	return -1;
 
     syslog(LOG_INFO, "succeeded: command=%s, book=%s/%s, bitmap-size=%d, \
@@ -1816,7 +1817,7 @@ command_XS(command, argument)
     if (current_book == NULL) {
 	syslog(LOG_INFO, "no current book: command=%s, host=%s(%s)",
 	    command, client_host_name, client_address);
-	if (write_string_all(accepted_out_file, "$<\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$<\n") <= 0)
 	    return -1;
 	return 0;
     }
@@ -1828,7 +1829,7 @@ command_XS(command, argument)
 	syslog(LOG_INFO, "bad direction sign: command=%s, position=%s, \
 host=%s(%s)",
 	    command, argument, client_host_name, client_address);
-	if (write_string_all(accepted_out_file, "$?\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$?\n") <= 0)
 	    return -1;
 	return 0;
     }
@@ -1841,7 +1842,7 @@ host=%s(%s)",
     if (!isxdigit(*(argument + 1)) || *end_p != ':') {
 	syslog(LOG_INFO, "bad position: command=%s, position=%s, host=%s(%s)",
 	    command, argument, client_host_name, client_address);
-	if (write_string_all(accepted_out_file, "$?\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$?\n") <= 0)
 	    return -1;
 	return 0;
     }
@@ -1854,7 +1855,7 @@ host=%s(%s)",
     if (!isxdigit(*offset_p) || *end_p != '\0') {
 	syslog(LOG_INFO, "bad position: command=%s, position=%s, host=%s(%s)",
 	    command, argument, client_host_name, client_address);
-	if (write_string_all(accepted_out_file, "$?\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$?\n") <= 0)
 	    return -1;
 	return 0;
     }
@@ -1868,7 +1869,7 @@ host=%s(%s)",
 host=%s(%s)",
 	    eb_error_message(error_code), command, current_book->name,
 	    current_subbook_name, argument, client_host_name, client_address);
-	if (write_string_all(accepted_out_file, "$1\n$$\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$1\n$$\n") <= 0)
 	    return -1;
 	return 0;
     }
@@ -1891,10 +1892,10 @@ host=%s(%s)",
      */
     if (error_code == EB_SUCCESS) {
 	sprintf(message, "$XS%x:%x\n", position.page, position.offset);
-	if (write_string_all(accepted_out_file, message) < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, message) <= 0)
 	    return -1;
     } else if (error_code == EB_ERR_END_OF_CONTENT) {
-	if (write_string_all(accepted_out_file, "$&\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$&\n") <= 0)
 	    return -1;
     } else {
 	syslog(LOG_INFO, "%s: command=%s, book=%s/%s, position=%s, \
@@ -1902,7 +1903,7 @@ host=%s(%s)",
 	    eb_error_message(error_code), command, current_book->name,
 	    current_subbook_name, argument, client_host_name,
 	    client_address);
-	if (write_string_all(accepted_out_file, "$&\n") < 0)
+	if (write_string_all(accepted_out_file, idle_timeout, "$&\n") <= 0)
 	    return -1;
 	return 0;
     }
